@@ -1,22 +1,50 @@
-setGeneric("plink", function(x, common, rescale, ability, method, weights, startvals, score=1, base.grp=1, symmetric=FALSE, rescale.com=FALSE, grp.names=NULL, mn.exclude=FALSE, dilation="ODL",  dim.order=NULL, ...) standardGeneric("plink"))
+setGeneric("plink", function(x, common, rescale, ability, method, weights.t, weights.f, startvals, score=1, base.grp=1, symmetric=FALSE, rescale.com=TRUE, grp.names=NULL, mn.exclude=0, dilation="ODL",  dim.order=NULL, ...) standardGeneric("plink"))
+
+
+
+setMethod("plink", signature(x="list", common="list"), function(x, common, rescale, ability, method, weights.t, weights.f, startvals, score, base.grp, symmetric, rescale.com, grp.names, mn.exclude, dilation, dim.order, ...) {
 
-setMethod("plink", signature(x="list", common="list"), function(x, common, rescale, ability, method, weights, startvals, score, base.grp, symmetric, rescale.com, grp.names, mn.exclude, dilation, dim.order, ...) {
 	x <- combine.pars(x, common, ...)
 	callGeneric()
+	
 })
 
-setMethod("plink", signature(x="list", common="matrix"), function(x, common, rescale, ability, method, weights, startvals, score, base.grp, symmetric, rescale.com, grp.names, mn.exclude, dilation, dim.order, ...) {
+
+
+setMethod("plink", signature(x="list", common="matrix"), function(x, common, rescale, ability, method, weights.t, weights.f, startvals, score, base.grp, symmetric, rescale.com, grp.names, mn.exclude, dilation, dim.order, ...) {
+
 	x <- combine.pars(x, common, ...)
 	callGeneric()
+	
 })
 
-setMethod("plink", signature(x="list", common="data.frame"), function(x, common, rescale, ability, method, weights, startvals, score, base.grp, symmetric, rescale.com, grp.names, mn.exclude, dilation, dim.order, ...) {
+
+
+setMethod("plink", signature(x="list", common="data.frame"), function(x, common, rescale, ability, method, weights.t, weights.f, startvals, score, base.grp, symmetric, rescale.com, grp.names, mn.exclude, dilation, dim.order, ...) {
+	
 	x <- combine.pars(x, common, ...)
 	callGeneric()
+	
 })
 
-setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, rescale, ability, method, weights, startvals, score, base.grp, symmetric, rescale.com, grp.names, mn.exclude, dilation, dim.order, ...) {
-	.CC <- function(startvals, to, from, dimensions, weights, score, transform, symmetric, mn.exclude, dilation, T, ...) {
+
+
+setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, rescale, ability, method, weights.t, weights.f, startvals, score, base.grp, symmetric, rescale.com, grp.names, mn.exclude, dilation, dim.order, ...) {
+
+	##   This is the function that will be minimized for the characteristic curve methods 
+	##   (both unidimensional and multidimensional)
+	.CC <- function(startvals, to, from, dimensions, weights.t, weights.f, score, transform, symmetric, mn.exclude, dilation, T, ...) {
+		
+		##   In general, the criterion that will minimized is Q = Q1+Q2
+		##   where Q1 corresponds to the differences in probabilities after
+		##   transforming the parameters on the FROM scale to the TO scale
+		##   and Q2 corresponds to the differences in probabilities after
+		##   transforming the parameters on the TO scale to the FROM scale.
+		##   When {symmetric}=TRUE, Q is minimized, but when
+		##   {symmetric}=FALSE, only Q1 will be minimized. 
+		
+		##   Identify optional arguments that might be passed
+		##   to the various functions used to compute response probabilities
 		dots <- list(...)
 		if (length(dots$D)) D <- dots$D else D <- 1
 		if (length(dots$D.drm)) D.drm <- dots$D.drm else D.drm <- D
@@ -25,81 +53,162 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 		if (length(dots$incorrect)) incorrect <- dots$incorrect else incorrect <- FALSE
 		if (length(dots$catprob)) catprob <- dots$catprob else catprob <- TRUE
 		
-		theta <- as.matrix(weights[[1]])
+		##   The theta values that will be used to compute response probabilities for Q1
+		theta.t <- as.matrix(weights.t[[1]])
+		
+		##   The theta values that will be used to compute response probabilities for Q1
+		theta.f <- as.matrix(weights.f[[1]])
+		
+		##   This object is used to store the TO scale parameters
+		##   after they have been transformed to the FROM scale
 		to.f <- to
+		
+		##   This object is used to store the FROM scale parameters
+		##   after they have been transformed to the TO scale
 		from.t <- from
+		
 		pm <- as.poly.mod(length(to@cat),to@model,to@items)
 		
 		if (dimensions==1) {
+			##   Assign the vector of startvals to meaningful objects
+			
+			##   This parameterization is for the Rasch model
+			##   where only the difficulties are transformed
 			if (length(startvals)==1) {
 				alpha <- 1
 				beta <- startvals[1]
+				
+			##   This parameterization is for all other models
 			} else {
 				alpha <- startvals[1]
 				beta <- startvals[2]
 			}
+			
+			##   Identify the nrm and mcm items
 			mcnr <- NULL
 			for (i in 1:length(to@model)) {
 				if (to@model[i]=="mcm"|to@model[i]=="nrm") mcnr <- c(mcnr, to@items[[i]])
 			}
-			# Transform the item parameters for the FROM scale
+			
+			##   Transform the FROM scale parameters onto the TO scale
+			##   using the linking constants alpha and beta
 			from.t@a <- as.matrix(from@a/alpha)
 			from.t@b <- as.matrix(alpha * from@b + beta)
 			from.t@b[mcnr,] <- from@b[mcnr,]-(beta/alpha)*from@a[mcnr,]
-			# Transform the item parameters for the TO scale
+			
+			##   Transform the FROM scale parameters onto the TO scale
+			##   using the linking constants alpha and beta
 			if (symmetric==TRUE) { 
 				to.f@a <- as.matrix(alpha*to@a)
 				to.f@b <- as.matrix((to@b-beta)/alpha)
 				to.f@b[mcnr,] <- to@b[mcnr,]+(beta*to@a[mcnr,])
-				prob.to.f <- .Mixed(to.f, theta, D.drm, D.gpcm, D.grm, incorrect, catprob)$p
-				prob.from <- .Mixed(from, theta, D.drm, D.gpcm, D.grm, incorrect, catprob)$p
+				
+				##   Compute response probabilities using the TO scale parameters that 
+				##   have been transformed onto the FROM scale
+				prob.to.f <- .Mixed(to.f, theta.f, D.drm, D.gpcm, D.grm, incorrect, catprob)$p
+				
+				##   Compute response probabilities using the untransformed FROM scale parameters
+				prob.from <- .Mixed(from, theta.f, D.drm, D.gpcm, D.grm, incorrect, catprob)$p
 			}
 		} else {
-			if (length(startvals==dimensions)) {
+		
+			if (length(startvals)==dimensions) {
+				##   This parameterization is for the multidimensional Rasch case
 				A <- diag(rep(1,dimensions))
+				m <- startvals
+				
 			} else {
+				##   This parameterization is used for all other models
+				##   although it differs depending on the specified dilation
 				if (dilation=="ODL") {
 					A <- matrix(startvals[1:(dimensions^2)],dimensions,dimensions)
+					m <- startvals[(dimensions^2+1):length(startvals)]
+					
+				##   For the LL and MIN approaches, the starting values need
+				##   to be rotated using the orthogonal rotation matrix T
+				##   prior to transforming any of the parameters. This makes it
+				##   so that parameters only capture changes in variability
 				} else if (dilation=="LL") {
 					A <- diag(rep(startvals[1],dimensions))%*%T
+					m <- startvals[2:length(startvals)]
+					
 				} else if (dilation=="MIN") {
 					A <- diag(startvals[1:dimensions])%*%T
+					m <- startvals[(dimensions+1):length(startvals)]
 				} 
+				
 			}
 			
-			# Transform the item parameters for the FROM scale
+			##   Transform the FROM scale parameters onto the TO scale
+			##   using the linking constants A and m
 			from.t@a <- from@a%*%ginv(A)
-			from.t@b <- from@b-matrix(from.t@a%*%startvals[((length(startvals)-dimensions)+1):length(startvals)],nrow(from@b),ncol(from@b))
+			from.t@b <- from@b-matrix(from.t@a %*% m, nrow(from@b),ncol(from@b))
 			
-			# Transform the item parameters for the TO scale
+			##   Transform the TO scale parameters onto the FROM scale
+			##   using the linking constants A and m
 			if (symmetric==TRUE) {
 				to.f@a <- to@a%*%A
-				to.f@b <- to@b+matrix(to@a%*%startvals[((length(startvals)-dimensions)+1):length(startvals)],nrow(to@b),ncol(to@b))
-				prob.to.f <- .Mixed(to.f, theta, D.drm, D.gpcm, D.grm, incorrect, catprob)$p
-				prob.from <- .Mixed(from, theta, D.drm, D.gpcm, D.grm, incorrect, catprob)$p
+				to.f@b <- to@b+matrix(to@ a%*% m, nrow(to@b),ncol(to@b))
+				
+				##   Compute response probabilities using the TO scale parameters that 
+				##   have been transformed onto the FROM scale
+				prob.to.f <- .Mixed(to.f, theta.f, D.drm, D.gpcm, D.grm, incorrect, catprob)$p
+				
+				##   Compute response probabilities using the untransformed FROM scale parameters
+				prob.from <- .Mixed(from, theta.f, D.drm, D.gpcm, D.grm, incorrect, catprob)$p
 			}
 		}
 		
-		tmp.to <- .Mixed(to, theta, D.drm, D.gpcm, D.grm, incorrect, catprob)
-		prob.to <- tmp.to$p
-		p.cat<- tmp.to$p.cat
-		prob.from.t <- .Mixed(from.t, theta, D.drm, D.gpcm, D.grm, incorrect, catprob)$p
+		##   Compute response probabilities using the untransformed TO scale parameters
+		tmp.to <- .Mixed(to, theta.t, D.drm, D.gpcm, D.grm, incorrect, catprob)
 		
-		cat <- rep(p.cat,p.cat)
+		##   Use the information in the {irt.prob} object tmp.to to 
+		##   determine the number of columns in the matrix of probabilities
+		##   associated with each item (i.e., the object p.cat)
+		prob.to <- tmp.to$p
+		p.cat <- tmp.to$p.cat
+		
+		##   Compute response probabilities using the FROM scale parameters that 
+		##   have been transformed onto the TO scale
+		prob.from.t <- .Mixed(from.t, theta.t, D.drm, D.gpcm, D.grm, incorrect, catprob)$p
+		
+		##   Create a vector identifying the response model associated
+		##   with each column of the matrix of probabilities
 		p.mod <- rep(tmp.to$p.mod,p.cat)
+		
+		##   Initialize the vector for the scoring function
+		##   that will be used for the Stocking-Lord method
 		scr <- NULL
+		
+		##   Create a set of default values for the scoring function
+		##   that range from 1 to Kj for K columns of probabilities for item j
 		for (h in 1:length(p.cat)) {
 			scr <- c(scr,seq(1,p.cat[h]))
 		}
+		
+		##   Create the equivalent of a scoring function for the Haebara method
+		##   This will be modified to exclude nrm and mcm items (if applicable)
+		scr.hb <- rep(1,length(scr))
+		
+		##   Use this if one of the default values (1 or 2) is used for the {score} argument
 		if (length(score)==1) {
+		
+			##   Use this if the lowest category should have a scoring weight of zero
 			if (score==2) {
 				scr <- scr-1
+				
+				##   If the argument {incorrect} equals FALSE, either explicitly or
+				##   by not including the argument, there will only be one column of 
+				##   probabilities for each dichotomous item. This formulation of the
+				##   scoring function will set the weight for these columns equal to 
+				##   zero. As such, set the weights for these items equal to one
+				cat <- rep(p.cat,p.cat)
 				scr[cat==1] <- 1
 			}
-			if (mn.exclude==TRUE) {
-				# Set the weights for all NRM and MCM items to zero (i.e., do not include these items)
-				scr[p.mod=="nrm"|p.mod=="mcm"] <- 0
-			}
+			
+			
+		##   Use this if the researcher supplies a vector of score weights for
+		##   all of the columns in p
 		} else {
 			if (length(score)==length(scr)) {
 				scr <- score
@@ -107,31 +216,41 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 				warning("The length of {score} does not match the number of response probabilities. Score was set to 1")
 			}
 		}
-		scr <- matrix(scr,nrow(prob.to),length(scr),byrow=TRUE)
-		# Compute sum of squares difference
-		W <- as.vector(weights[[2]])
+		
+		##   If nrm and mcm items are supposed to be excluded from the
+		##   estimation of linking constants, set the probabilities for all responses
+		##   for all of these items on both tests to zero
+		if (mn.exclude %in% c(2,3)) {
+			scr[p.mod=="nrm"|p.mod=="mcm"] <- 0
+			scr.hb[p.mod=="nrm"|p.mod=="mcm"] <- 0
+		}
+		
+		##   Compute sum of squares difference for the 
+		##   Haebara and Stocking-Lord methods
+		W1 <- as.vector(weights.t[[2]])
+		W2 <- as.vector(weights.f[[2]])
 		if (transform=="HB") {
-			L1 <- ncol(prob.to)*sum(W)
-			Q1 <- W*(prob.to-prob.from.t)^2
-			Q1 <- sum(apply(Q1,1,sum))
+			L1 <- ncol(prob.to)*sum(W1)
+			Q1 <- W1*(prob.to-prob.from.t)^2
+			Q1 <- sum(Q1 %*% scr.hb)
 			if (symmetric==TRUE) {
-				L2 <- ncol(prob.to)*sum(W)
-				Q2 <- W*(prob.from-prob.to.f)^2
-				Q2 <- sum(apply(Q2,1,sum))
+				L2 <- ncol(prob.to)*sum(W2)
+				Q2 <- W2*(prob.from-prob.to.f)^2
+				Q2 <- sum(Q2 %*% scr.hb)
 				SS <- (Q1/L1)+(Q2/L2)
 			} else {
 				SS <- Q1/L1
 			}
 		} else if (transform=="SL") {
-			L1 <- sum(W)
-			TCC.to <- apply(scr*prob.to, 1, sum)
-			TCC.from <- apply(scr*prob.from.t, 1, sum)
-			Q1 <-sum(W*(TCC.to - TCC.from)^2)
+			L1 <- sum(W1)
+			TCC.to <- prob.to %*% scr
+			TCC.from <- prob.from.t %*% scr
+			Q1 <-sum(W1*(TCC.to - TCC.from)^2)
 			if (symmetric==TRUE) {
-				L2 <- sum(W)
-				TCC.to <- apply(scr*prob.to.f, 1, sum)
-				TCC.from <- apply(scr*prob.from, 1, sum)
-				Q2 <-sum(W*(TCC.from- TCC.to)^2)
+				L2 <- sum(W2)
+				TCC.to <- prob.to.f %*% scr
+				TCC.from <- prob.from %*% scr
+				Q2 <-sum(W2*(TCC.from- TCC.to)^2)
 				SS <- (Q1/L1)+(Q2/L2)
 			} else {
 				SS <- Q1/L1
@@ -140,22 +259,28 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 		return(SS)
 	}
 	
+	
+	
+	
+	##   This function is used to compute either the  orthogonal or 
+	##   oblique rotation matrix for the multidimensional approaches
 	.Rotate <- function(to, from, orthogonal) {
 		if (orthogonal==TRUE) {
-			# Orthogonal procrustes rotation
+			##   Orthogonal procrustes rotation
 			M <- svd(t(to)%*%from)
 			T <- M$v%*%t(M$u)
+			
 		} else {
-			# Oblique procrustes rotation
+			##   Oblique procrustes rotation
 			T <- ginv(t(from) %*% from) %*% t(from) %*% to
 		}
 		
 		TF <- T
 		mn <- sum((to-from%*%ginv(T))^2)
 		for (i in 1:3) {
-			# Due to indeterminacies in the sign of eigenvectors (or the orientation), 
-			# adjust the rotation matrix to get the matrix that truly minimizes the 
-			# difference between the two sets of slopes
+			##   Due to indeterminacies in the sign of eigenvectors (or the orientation), 
+			##   adjust the rotation matrix to get the matrix that truly minimizes the 
+			##   difference between the two sets of slopes
 			if (i==1) T1 <- t(T)
 			if (i==2) T1 <- T*-1
 			if (i==3) T1 <- t(T*-1)
@@ -168,7 +293,12 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 		return(T)
 	}
 	
+	
+	
+	
+	##   This is a stripped down version of the {mixed} function
 	.Mixed <- function(x, theta, D.drm, D.gpcm, D.grm, incorrect, catprob) {
+		
 		mod <- x@model
 		p <- NULL
 		p.cat <- NULL
@@ -186,35 +316,41 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 		return(list(p=p,p.cat=p.cat,p.mod=p.mod))
 	}
 	
+	
+	
+	##   This is a stripped down version of the {drm} function
 	.Drm <- function(x, theta, D.drm, incorrect) {
-		items <- x@items$drm
+		
 		dimensions <- x@dimensions
+		items <- x@items$drm
 		n <- length(items)
-		a <- x@a[items,1:dimensions]
+		a <- as.matrix(x@a[items,1:dimensions])
+		if (n==1) a <- t(a)
 		b <- x@b[items,1]
 		c <- x@c[items,1]
-		# Compute item probabilities
+		
 		p <- NULL
-		if (dimensions==1) {
-			for (i in 1:length(b)) {
+		for (i in 1:length(b)) {
+			if (dimensions==1) {
 				cp <- c[i]+(1-c[i])/(1+exp(-D.drm*a[i]*(theta-b[i])))
-				if (incorrect==TRUE) p <- cbind(p,(1-cp),cp) else p <- cbind(p,cp)
-			}
-		} else {
-			a <- a*D.drm
-			for (i in 1:length(b)) {
+			} else {
 				cp <- c[i]+(1-c[i])/(1+exp(-(theta %*% a[i,]+b[i])))
-				if (incorrect==TRUE) p <- cbind(p,(1-cp),cp) else p <- cbind(p,cp)
+				
 			}
+			if (incorrect==TRUE) p <- cbind(p,(1-cp),cp) else p <- cbind(p,cp)
 		}
 		if (incorrect==TRUE) cat <- rep(2,n) else cat <- rep(1,n)
 		mod <- rep("drm",n)
 		return(list(p=p,cat=cat,mod=mod))
 	}
 	
+	
+	
+	##   This is a stripped down version of the {gpcm} function
 	.Gpcm <- function(x, theta, D.gpcm) {
-		items <- x@items$gpcm
+		
 		dimensions <- x@dimensions
+		items <- x@items$gpcm
 		n <- length(items)
 		a <- as.matrix(x@a[items,1:dimensions])
 		b <- as.matrix(x@b[items,])
@@ -223,50 +359,45 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 			b <- t(b)
 		}
 		cat <- x@cat[items]
-		# Compute category probabilities
+		
 		p <- NULL 
-		if (dimensions==1) {
-			for (i in 1:nrow(b)) {
-				dif <- 0 # Difference between subsequent step parameters
-				den <- NULL # Compute the denominator
-				for (k in 0:(cat[i]-1)) {
-					if (k==1) dif <- b[i,k] else if (k>1) dif <- dif+b[i,k]
+		for (i in 1:nrow(b)) {
+			dif <- 0 
+			den <- NULL 
+			
+			for (k in 0:(cat[i]-1)) {
+				if (k>=1) dif <- dif+b[i,k]
+				if (dimensions==1) {
 					d <- exp(D.gpcm*a[i]*(k*theta-dif))
-					den <- cbind(den, d)
-				}
-				den <- apply(den,1,sum)
-				dif <- 0
-				for (k in 0:(cat[i]-1)) {
-					if (k==1) dif <- b[i,k] else if (k>1) dif <- dif+b[i,k]
-					cp <- (exp(D.gpcm*a[i]*(k*theta-dif)))/den
-					p <- cbind(p,cp)
+				} else {
+					d <- exp(k*(theta %*% a[i,])+dif)
 				}
+				den <- cbind(den, d)
 			}
-		} else {
-			for (i in 1:nrow(b)) {
-				dif <- 0 # Difference between subsequent step parameters
-				den <- NULL # Compute the denominator
-				for (k in 0:(cat[i]-1)) {
-					if (k>=1) dif <- dif+b[i,k]
-					d <- exp(D.gpcm*k*(theta %*% a[i,])+dif)
-					den <- cbind(den, d)
-				}
-				den <- apply(den,1,sum)
-				dif <- 0
-				for (k in 0:(cat[i]-1)) {
-					if (k>=1) dif <- dif+b[i,k]
-					cp <- exp(D.gpcm*k*(theta %*% a[i,])+dif)/den
-					p <- cbind(p,cp)
+			den <- apply(den,1,sum)
+			
+			dif <- 0
+			for (k in 0:(cat[i]-1)) {
+				if (k>=1) dif <- dif+b[i,k]
+				if (dimensions==1) {
+					cp <- (exp(D.gpcm*a[i]*(k*theta-dif)))/den
+				} else {
+					cp <- exp(k*(theta %*% a[i,])+dif)/den
 				}
+				p <- cbind(p,cp)
 			}
 		}
 		mod <- rep("gpcm",n)
 		return(list(p=p,cat=cat,mod=mod))
 	}
 	
+	
+	
+	##   This is a stripped down version of the {grm} function
 	.Grm <- function(x, theta, catprob, D.grm) {
-		items <- x@items$grm
+	
 		dimensions <- x@dimensions
+		items <- x@items$grm
 		n <- length(items)
 		a <- as.matrix(x@a[items,1:dimensions])
 		b <- as.matrix(x@b[items,])
@@ -276,53 +407,46 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 		}
 		cat <- x@cat[items]
 		p <- NULL 
-		if (dimensions==1) {
-			if (catprob==TRUE) { 
-				for (i in 1:n) {
-					ct <- cat[i]-1
+		
+		##   Compute category probabilities
+		if (catprob==TRUE) { 
+			for (i in 1:n) {
+				ct <- cat[i]-1
+				if (dimensions==1) {
 					cp <- 1-1/(1+exp(-D.grm*a[i]*(theta-b[i,1])))
-					p <- cbind(p, cp)
-					for (k in 1:ct) {
+				} else {
+					cp <- 1-1/(1+exp(-(theta %*% a[i,]+b[i,1])))
+				}
+				p <- cbind(p, cp)
+				
+				for (k in 1:ct) {
+					if (dimensions==1) {
 						if (k<ct) {
 							cp <- (1/(1+exp(-D.grm*a[i]*(theta-b[i,k]))))-(1/(1+exp(-D.grm*a[i]*(theta-b[i,k+1]))))
 						} else if (k==ct) {
 							cp <- 1/(1+exp(-D.grm*a[i]*(theta-b[i,k])))
 						}
-						p <- cbind(p, cp)
-					}
-				}
-			# Compute cumulative probabilities
-			} else if (catprob==FALSE) { 
-				for (i in 1:n) {
-					for (k in 1:(cat[i]-1)) {
-						cp <- 1/(1+exp(-D.grm*a[i]*(theta-b[i,k])))
-						p <- cbind(p, cp)
-					}
-				}
-			}
-		} else {
-			# Compute category probabilities
-			if (catprob==TRUE) { 
-				for (i in 1:n) {
-					ct <- cat[i]-1
-					cp <- 1-1/(1+exp(-D.grm*(theta %*% a[i,])+b[i,1]))
-					p <- cbind(p, cp)
-					for (k in 1:ct) {
+					} else {
 						if (k<ct) {
 							cp <- (1/(1+exp(-(theta %*% a[i,]+b[i,k]))))-(1/(1+exp(-(theta %*% a[i,]+b[i,k+1]))))
 						} else if (k==ct) {
 							cp <- 1/(1+exp(-(theta %*% a[i,]+b[i,k])))
 						}
-						p <- cbind(p, cp)
 					}
+					p <- cbind(p, cp)
 				}
-			# Compute cumulative probabilities
-			} else if (catprob==FALSE) { 
-				for (i in 1:n) {
-					for (k in 1:(cat[i]-1)) {
+			}
+			
+		# Compute cumulative probabilities
+		} else if (catprob==FALSE) { 
+			for (i in 1:n) {
+				for (k in 1:(cat[i]-1)) {
+					if (dimensions==1) {
+						cp <- 1/(1+exp(-D.grm*a[i]*(theta-b[i,k])))
+					} else {
 						cp <- 1/(1+exp(-(theta %*% a[i,]+b[i,k])))
-						p <- cbind(p, cp)
 					}
+					p <- cbind(p, cp)
 				}
 			}
 		}
@@ -331,9 +455,13 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 		return(list(p=p,cat=cat,mod=mod))
 	}
 	
+	
+	
+	##   This is a stripped down version of the {mcm} function
 	.Mcm <- function(x, theta) {
-		items <- x@items$mcm
+	
 		dimensions <- x@dimensions
+		items <- x@items$mcm
 		n <- length(items)
 		a <- as.matrix(x@a[items,]) 
 		b <- as.matrix(x@b[items,])
@@ -344,10 +472,10 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 			c <- t(c)
 		}
 		cat <- x@cat[items]
-		# Compute category probabilities
+		
 		p <- NULL 
 		for (i in 1:n) {
-			den <- NULL # Compute the denominator
+			den <- NULL
 			a1 <- a[i,][!is.na(a[i,])]
 			b1 <- b[i,][!is.na(b[i,])]
 			c1 <- c[i,][!is.na(c[i,])]
@@ -373,9 +501,13 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 		return(list(p=p,cat=cat,mod=mod))
 	}
 	
+	
+	
+	##   This is a stripped down version of the {nrm} function
 	.Nrm <- function(x, theta) {
-		items <- x@items$nrm
+	
 		dimensions <- x@dimensions
+		items <- x@items$nrm
 		n <- length(items)
 		a <- as.matrix(x@a[items,]) 
 		b <- as.matrix(x@b[items,])
@@ -385,10 +517,9 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 		}
 		cat <- x@cat[items]
 		
-		# Compute category probabilities
 		p <- NULL 
 		for (i in 1:n) {
-			den <- NULL # Compute the denominator
+			den <- NULL
 			a1 <- a[i,][!is.na(a[i,])]
 			b1 <- b[i,][!is.na(b[i,])]
 			for (k in 1:cat[i]) {
@@ -409,36 +540,57 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 		return(list(p=p,cat=cat,mod=mod))
 	}
 	
+	
+	
+	##   The bult-in SD function R uses n-1 in the denominator
+	##   This function uses n in the denominator
 	.sd <- function(x) {
 		z <- x[!is.na(x)]
 		out <- sqrt(sum((z-mean(z))^2)/length(z))
 		return(out)
 	}
 	
+	
+	
+	##   This function computes descriptive statistics for the common item parameters
 	.Descriptives <- function(a1, a2, b1, b2, c1, c2, pm, cat, mn.exclude, dimensions) {
-	# Compute descriptive statistics for the common item parameters
 	
 	if (dimensions==1) {
-		# Transform b for mcm and nrm items
+		##   Transform the category parameters for mcm and nrm items in group 1
 		if (ncol(a1)!=ncol(b1)) {
 			b1r <- as.matrix(-b1/matrix(a1,nrow(a1),ncol(b1)))
 		} else {
 			b1r <- as.matrix(-b1/a1)
 		}
+		
+		##   Transform the category parameters for mcm and nrm items in group 2
 		if (ncol(a2)!=ncol(b2)) {
 			b2r <- as.matrix(-b2/matrix(a2,nrow(a2),ncol(b2)))
 		} else {
 			b2r <- as.matrix(-b2/a2)
 		}
-		b1r[b1r==Inf] <- 0
-		b2r[b2r==Inf] <- 0
-		b1r[b1r==-Inf] <- 0
-		b2r[b2r==-Inf] <- 0
 		
+		##   In the above transformations it is possible for some slope parameters
+		##   to equal zero. As such the transformation will return values equal to
+		##   infinity or negative infinity. These values should be set to NA.
+		b1r[b1r== Inf] <- NA
+		b2r[b2r== Inf] <- NA
+		b1r[b1r== -Inf] <- NA
+		b2r[b2r== -Inf] <- NA
+		
+		##   Initialize an object to store the descriptives
 		descrip <- NULL
+		
+		##   Initialize an object to identify MCM or NRM items
 		mcnr <- NULL
+		
+		##   Vector of response models for the common items
 		pm.mod <- pm@model
+		
+		##   List of items associated with each model in pm.mod
 		pm.it <- pm@items
+		
+		##   Extract the item parameters for each model in each group
 		for (j in 1:length(pm.mod)) {
 			a1k <- a1[pm.it[[j]],]
 			a2k <- a2[pm.it[[j]],]
@@ -449,11 +601,14 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 			b1kr <- b1r[pm.it[[j]],]
 			b2kr <- b2r[pm.it[[j]],]
 			
+			##   Identify the number of each type of parameter, and compute the 
+			##   means and SDs of each parameter type for both groups
 			a <- c(length(a1k[!is.na(a1k)]),mean(a1k,na.rm=TRUE),mean(a2k,na.rm=TRUE),.sd(a1k),.sd(a2k))
 			b <- c(length(b1k[!is.na(b1k)]),mean(b1k,na.rm=TRUE),mean(b2k,na.rm=TRUE),.sd(b1k),.sd(b2k))
 			c <- c(length(c1k[!is.na(c1k)]),mean(c1k,na.rm=TRUE),mean(c2k,na.rm=TRUE),.sd(c1k),.sd(c2k))
 			bd <- c(length(b1kr[!is.na(b1kr)]),mean(b1kr,na.rm=TRUE),mean(b2kr,na.rm=TRUE),.sd(b1kr),.sd(b2kr))
 			
+			##   Compile these descriptives for each model
 			if (pm.mod[j]=="drm" ) {
 				des <- data.frame(cbind(a,b,c))
 				colnames(des) <- c("a","b","c")
@@ -473,12 +628,17 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 			descrip[[j]] <- round(des,6)
 		}
 		
-		if (sum((mod=="nrm"|mod=="mcm")+0)==length(mod)) mn.exclude <- FALSE
+		##   Check to see if all of the common items are modeled using the 
+		##   NRM or MCM. If so, do not exclude any items from the computation
+		##   of the descriptives (which will ultimately be used with the moment methods)
+		if (sum((pm.mod=="nrm"|pm.mod=="mcm")+0)==length(pm.mod)) mn.exclude <- 0
 		
-		if (mn.exclude==FALSE) {
+		
+		if (mn.exclude %in% c(0,2)) {
 			b1[mcnr,] <- b1r[mcnr,] 
 			b2[mcnr,] <- b2r[mcnr,] 
 		} else {
+			##   Remove NRM and/or MCM item parameters
 			if (!is.null(mcnr)) {
 				a1 <- a1[-mcnr,]
 				a2 <- a2[-mcnr,]
@@ -486,18 +646,35 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 				b2 <- b2[-mcnr,]
 			}
 		}
+		
+		##   Identify the number of each type of parameter, and compute the 
+		##   means and SDs of each parameter type for both groups
 		a <- c(length(a1[!is.na(a1)]),mean(a1,na.rm=TRUE),mean(a2,na.rm=TRUE),.sd(a1),.sd(a2))
 		b <- c(length(b1[!is.na(b1)]),mean(b1,na.rm=TRUE),mean(b2,na.rm=TRUE),.sd(b1),.sd(b2))
+		
+		##   Compile these descriptives for all "included" common items
 		desall <- round(cbind(a,b),6)
 		colnames(desall) <- c("a","b")
 		rownames(desall) <- c("N Pars:","Mean: To","Mean: From","SD: To","SD: From")
 		descrip[[length(descrip)+1]] <- desall
 		names(descrip) <- c(pm.mod,"all")
+		
 	} else if (dimensions>1) {
+		##   Initialize an object to store the descriptives
 		descrip <- NULL
+		
+		##   Initialize an object to identify MCM or NRM items
+		mcnr <- NULL
+		
+		##   Vector of response models for the common items
 		pm.mod <- pm@model
+		
+		##   List of items associated with each model in pm.mod
 		pm.it <- pm@items
+		
 		a1.all <- a2.all <- vector("list",dimensions)
+		
+		##   Extract the item parameters for each model in each group
 		for (j in 1:length(pm.mod)) {
 			a1k <- a1[pm.it[[j]],]
 			a2k <- a2[pm.it[[j]],]
@@ -527,15 +704,23 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 				}
 			}
 			
+			##   Identify the number of each type of parameter, and compute the 
+			##   means and SDs of each parameter type for both groups
 			b <- c(length(b1k[!is.na(b1k)]),mean(b1k,na.rm=TRUE),mean(b2k,na.rm=TRUE),.sd(b1k),.sd(b2k))
 			c <- c(length(c1k[!is.na(c1k)]),mean(c1k,na.rm=TRUE),mean(c2k,na.rm=TRUE),.sd(c1k),.sd(c2k))
+			
+			##   Compute MDISC and MDIF for all common items for both groups
 			mdisc1 <- sqrt(apply(a1k^2,1,sum,na.rm=TRUE))
 			mdisc2 <- sqrt(apply(a2k^2,1,sum,na.rm=TRUE))
 			mdif1 <- -b1k/mdisc1
 			mdif2 <- -b2k/mdisc2
+			
+			##   Identify the number MDISC and MDIF parameters, and compute the 
+			##   means and SDs of each for both groups
 			mdc <- c(length(mdisc1[!is.na(mdisc1)]),mean(mdisc1,na.rm=TRUE),mean(mdisc2,na.rm=TRUE),.sd(mdisc1),.sd(mdisc2))
 			mdf <- c(length(mdif1[!is.na(mdif1)]),mean(mdif1,na.rm=TRUE),mean(mdif2,na.rm=TRUE),.sd(mdif1),.sd(mdif2))
 			
+			##   Compile these descriptives for each model
 			if (pm.mod[j]=="drm" ) {
 				des <- data.frame(cbind(a,b,c,mdc,mdf))
 				colnames(des) <- c(paste("a",1:dimensions,sep=""),"d","c","MDISC","MDIF")
@@ -546,7 +731,8 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 				des <- data.frame(cbind(a,b,mdc,mdf))
 				colnames(des) <- c(paste("a",1:dimensions,sep=""),"d","MDISC","MDIF")
 			}
-			if (dim.flag==TRUE) des <- des[,-2]
+			
+			if (dim.flag==TRUE) des <- des[,-2]  ##  WHAT IS THIS?
 			colnames(des)[1] <- "a"
 			rownames(des) <- c("N Pars:","Mean: To","Mean: From","SD: To","SD: From")
 			descrip[[j]] <- round(des,6)
@@ -576,6 +762,9 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 	return(descrip)
 	}
 	
+	
+	
+	
 	### START PLINK ###
 	ng <- x@groups
 	md <- max(x@dimensions)
@@ -691,7 +880,9 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 		c1 <- tmp1@c
 		c2 <- tmp2@c
 		
-		if (missing(weights)) wgt<- as.weight(dimensions=dimensions) else wgt <- weights # FIX THIS
+		if (missing(weights.t)) wgt.t <- as.weight(dimensions=dimensions) else wgt.t <- weights.t 
+		if (missing(weights.f)) wgt.f <- wgt.t else wgt.f <- weights.f 
+		
 		#if (is.list(weights[[1]])) wgt <- weights[[i]] else 
 		
 		if (missing(method)) {
@@ -709,10 +900,12 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 		chk.sl <- TRUE
 		if (sum((mod=="nrm"|mod=="mcm")+0)==length(mod)) {
 			chk.sl <- FALSE
-			if (mn.exclude==TRUE) {
-				mn.exclude <- FALSE
-				cat("All items were used to compute the Mean/Mean and Mean/Sigma linking constants\n")
+			meth <- NULL
+			if ("MS" %in% method) meth <- "Mean/Sigma"
+			if ("MM" %in% method) {
+				if (length(meth)) meth <- "Mean/Sigma and Mean/Mean" else meth <- "Mean/Mean"
 			}
+			if (length(meth)) cat(paste("All items were used to compute the",meth,"linking constants\n"))
 		}
 		
 		# Perform the Calibration
@@ -723,11 +916,15 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 		if (dimensions==1) {
 			# Compute the mean/mean and mean/sigma constants
 			mm <- ms <- NULL
-			A1 <- mean(a2,na.rm=TRUE)/mean(a1,na.rm=TRUE)
-			A2 <- .sd(b1)/.sd(b2)
+			A1 <- descrip$all[3,1]/descrip$all[2,1]
+			A2 <- descrip$all[4,2]/descrip$all[5,2]
+			#A1 <- mean(a2,na.rm=TRUE)/mean(a1,na.rm=TRUE)
+			#A2 <- .sd(b1)/.sd(b2)
 			if (rasch.flag==TRUE) A2 <- a1[1]
-			B1 <- mean(b1,na.rm=TRUE)-A1*mean(b2,na.rm=TRUE)
-			B2 <- mean(b1,na.rm=TRUE)-A2*mean(b2,na.rm=TRUE)
+			B1 <- descrip$all[2,2]-A1*descrip$all[3,2]
+			B2 <- descrip$all[2,2]-A2*descrip$all[3,2]
+			#B1 <- mean(b1,na.rm=TRUE)-A1*mean(b2,na.rm=TRUE)
+			#B2 <- mean(b1,na.rm=TRUE)-A2*mean(b2,na.rm=TRUE)
 			mm <- round(c(A1,B1),6)
 			ms <- round(c(A2,B2),6)
 			names(mm) <- names(ms) <- c("A","B")
@@ -738,7 +935,7 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 			
 			# Determine starting values for the characteristic curve methods
 			if (missing(startvals)) {
-				startvals <- mm
+				startvals <- ms
 			} else {
 				if (is.character(startvals)) {
 					if (toupper(startvals)=="MM") startvals <- mm
@@ -825,7 +1022,7 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 		
 		# Haebara Method
 		if ("HB" %in% method) {
-			hb <- nlminb(startvals, .CC, to=tmp1, from=tmp2, dimensions=dimensions, weights=wgt, score=score, transform="HB", symmetric=symmetric, mn.exclude=mn.exclude, dilation=dilation, T=T, lower=lower, upper=upper, ...)
+			hb <- nlminb(startvals, .CC, to=tmp1, from=tmp2, dimensions=dimensions, weights.t=wgt.t, weights.f=wgt.f, score=score, transform="HB", symmetric=symmetric, mn.exclude=mn.exclude, dilation=dilation, T=T, lower=lower, upper=upper, ...)
 			if (dimensions==1) { 
 				if (rasch.flag==TRUE) hb$par <- c(1,hb$par)
 				names(hb$par) <- c("A","B")
@@ -868,7 +1065,7 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 		# Stocking-Lord Method
 		if ("SL" %in% method) {
 			if (chk.sl==TRUE) {
-				sl <- nlminb(startvals, .CC, to=tmp1, from=tmp2, dimensions=dimensions, weights=wgt, score=score, transform="SL", symmetric=symmetric, mn.exclude=mn.exclude, dilation=dilation, T=T, lower=lower, upper=upper, ...)
+				sl <- nlminb(startvals, .CC, to=tmp1, from=tmp2, dimensions=dimensions, weights.t=wgt.t, weights.f=wgt.f, score=score, transform="SL", symmetric=symmetric, mn.exclude=mn.exclude, dilation=dilation, T=T, lower=lower, upper=upper, ...)
 				if (dimensions==1) { 
 					if (rasch.flag==TRUE) sl$par <- c(1,sl$par)
 					names(sl$par) <- c("A","B")
@@ -951,6 +1148,7 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 		
 		out.pars <- vector("list",ng)
 		out.ability <- vector("list",ng)
+		
 		for (i in 1:ng) {
 			mcnr <- NULL
 			pm <- x@poly.mod[[i]]@model
@@ -987,11 +1185,11 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 					
 					if (length(do1)==1) {
 						tmp1@a[,do1] <- tmp1@a[,do1]*ginv(tmp.con[[j]][1])
-						tmp1@b <- tmp1@b-tmp1@a[,do1]*tmp.con[[j]][2]
+						tmp1@b <- tmp1@b-matrix(tmp1@a[,do1]*tmp.con[[j]][2],nrow(tmp1@b),ncol(tmp1@b))
 						if (!missing(ability)) tmpa <- tmp.con[[j]][1]*tmpa + tmp.con[[j]][2]
 					} else {
 						tmp1@a[,do1] <- tmp1@a[,do1]%*%ginv(tmp.con[[j]]$A)
-						tmp1@b <- tmp1@b-tmp1@a[,do1]%*%tmp.con[[j]]$m
+						tmp1@b <- tmp1@b-matrix(tmp1@a[,do1]%*%tmp.con[[j]]$m,nrow(tmp1@b),ncol(tmp1@b))
 						if (!missing(ability)) tmpa <- t(tmp.con[[j]]$A%*%t(tmpa[,do1]) + tmp.con[[j]]$m)
 					}
 				}
@@ -1009,8 +1207,11 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 			if (is.matrix(com)) com <- list(com)
 			for (i in (base.grp-1):1) {
 				if (base.grp==1) break
-				out.pars[[i]]@b[com[[i]][,1],] <- out.pars[[i+1]]@b[com[[i]][,2],]
-				out.pars[[i]]@c[com[[i]][,1],] <- out.pars[[i+1]]@c[com[[i]][,2],]
+				b.min <- min(c(ncol(out.pars[[i]]@b),ncol(out.pars[[i+1]]@b)))
+				c.min <- min(c(ncol(out.pars[[i]]@c),ncol(out.pars[[i+1]]@c)))
+				
+				out.pars[[i]]@b[com[[i]][,1],1:b.min] <- out.pars[[i+1]]@b[com[[i]][,2],1:b.min]
+				out.pars[[i]]@c[com[[i]][,1],1:c.min] <- out.pars[[i+1]]@c[com[[i]][,2],1:c.min]
 				if (dimensions==1) {
 					out.pars[[i]]@a[com[[i]][,1],] <- out.pars[[i+1]]@a[com[[i]][,2],]
 				} else {
@@ -1023,8 +1224,11 @@ setMethod("plink", signature(x="irt.pars", common="ANY"), function(x, common, re
 			
 			for (i in base.grp:(ng-1)) {
 				if (base.grp==ng) break
-				out.pars[[i+1]]@b[com[[i]][,2],] <- out.pars[[i]]@b[com[[i]][,1],]
-				out.pars[[i+1]]@c[com[[i]][,2],] <- out.pars[[i]]@c[com[[i]][,1],]
+				b.min <- min(c(ncol(out.pars[[i]]@b),ncol(out.pars[[i+1]]@b)))
+				c.min <- min(c(ncol(out.pars[[i]]@c),ncol(out.pars[[i+1]]@c)))
+				
+				out.pars[[i+1]]@b[com[[i]][,2],1:b.min] <- out.pars[[i]]@b[com[[i]][,1],1:b.min]
+				out.pars[[i+1]]@c[com[[i]][,2],1:c.min] <- out.pars[[i]]@c[com[[i]][,1],1:c.min]
 				if (dimensions==1) {
 					out.pars[[i+1]]@a[com[[i]][,2],] <- out.pars[[i]]@a[com[[i]][,1],]
 				} else {
